@@ -1,15 +1,15 @@
 ;;; mmm-mode.el --- Allow Multiple Major Modes in a buffer
 
-;; Copyright (C) 1999, 2004 by Michael Abraham Shulman
-;; Copyright (C) 2013-2015 by Dmitry Gutov
+;; Copyright (C) 1999-2004, 2012-2015, 2018  Free Software Foundation, Inc.
 
 ;; Emacs Lisp Archive Entry
 ;; Package: mmm-mode
-;; Author: Michael Abraham Shulman <viritrilbia@users.sourceforge.net>
+;; Author: Michael Abraham Shulman <viritrilbia@gmail.com>
 ;; Maintainer: Dmitry Gutov <dgutov@yandex.ru>
 ;; URL: https://github.com/purcell/mmm-mode
 ;; Keywords: convenience, faces, languages, tools
-;; Version: 0.5.4
+;; Version: 0.5.6
+;; Package-Requires: ((cl-lib "0.2"))
 
 ;;{{{ GPL
 
@@ -32,58 +32,58 @@
 
 ;;; Commentary:
 
-;;; MMM Mode is a minor mode that allows multiple major modes to
-;;; coexist in a single buffer. Refer to the documentation of the
-;;; function `mmm-mode' for more detailed information. This file
-;;; contains mode on/off functions and the mode keymap, but mostly
-;;; just loads all the subsidiary files.
+;; MMM Mode is a minor mode that allows multiple major modes to
+;; coexist in a single buffer. Refer to the documentation of the
+;; function `mmm-mode' for more detailed information. This file
+;; contains mode on/off functions and the mode keymap, but mostly
+;; just loads all the subsidiary files.
 
-;;{{{ Parameter Naming
+;{{{ Parameter Naming
 
-;;; Since version 0.3.7, I've tried to use a uniform scheme for naming
-;;; parameters. Here's a brief summary.
+;; Since version 0.3.7, I've tried to use a uniform scheme for naming
+;; parameters. Here's a brief summary.
 
-;;; BEG and END refer to the beginning and end of a region.
-;;; FRONT and BACK refer to the respective delimiters of a region.
-;;; FRONT- and BACK-OFFSET are the offsets from delimiter matches.
-;;; FRONT-BEG through BACK-END are the endings of the delimiters.
-;;; START and STOP bound actions, like searching, fontification, etc.
+;; BEG and END refer to the beginning and end of a region.
+;; FRONT and BACK refer to the respective delimiters of a region.
+;; FRONT- and BACK-OFFSET are the offsets from delimiter matches.
+;; FRONT-BEG through BACK-END are the endings of the delimiters.
+;; START and STOP bound actions, like searching, fontification, etc.
 
-;;}}}
-;;{{{ CL and Parameters
+;}}}
+;{{{ CL and Parameters
 
-;;; Keyword parameters can be nice because it makes it easier to see
-;;; what's getting passed as what. But I try not to use them in user
-;;; functions, because CL doesn't make good documentation strings.
-;;; Similarly, any hook or callback function can't take keywords,
-;;; since Emacs as a whole doesn't use them. And for small parameter
-;;; lists, they are overkill. So I use them only for a large number of
-;;; optional parameters, such as `mmm-make-region'.
+;; Keyword parameters can be nice because it makes it easier to see
+;; what's getting passed as what. But I try not to use them in user
+;; functions, because CL doesn't make good documentation strings.
+;; Similarly, any hook or callback function can't take keywords,
+;; since Emacs as a whole doesn't use them. And for small parameter
+;; lists, they are overkill. So I use them only for a large number of
+;; optional parameters, such as `mmm-make-region'.
 
-;;; An exception is the various submode class application functions,
-;;; which all take all their arguments as keywords, for consistency
-;;; and so the classes alist looks nice.
+;; An exception is the various submode class application functions,
+;; which all take all their arguments as keywords, for consistency
+;; and so the classes alist looks nice.
 
-;;; When using keyword arguments, defaults should *always* be supplied
-;;; in all arglists. (This pertains mostly to :start and :stop
-;;; arguments, usually defaulting to (point-min) and (point-max)
-;;; respectively.) `mmm-save-keywords' should only be used for lists
-;;; with more than four arguments, such as in `mmm-ify-by-regexp'.
+;; When using keyword arguments, defaults should *always* be supplied
+;; in all arglists. (This pertains mostly to :start and :stop
+;; arguments, usually defaulting to (point-min) and (point-max)
+;; respectively.) `mmm-save-keywords' should only be used for lists
+;; with more than four arguments, such as in `mmm-ify-by-regexp'.
 
-;;; In general, while I have no qualms about using things from CL like
-;;; `mapl', `loop' and `destructuring-bind', I try not to use `defun*'
-;;; more than I have to. For one, it sometimes makes bad documentation
-;;; strings. Furthermore, to a `defun'ned function, a nil argument is
-;;; the same as no argument, so it will use its (manual) default, but
-;;; to a `defun*'ned function, a nil argument *is* the argument, so
-;;; any default specified in the arglist will be ignored. Confusion of
-;;; this type should be avoided when at all possible.
+;; In general, while I have no qualms about using things from CL like
+;; `cl-mapl', `cl-loop' and `cl-destructuring-bind', I try not to use `cl-defun'
+;; more than I have to. For one, it sometimes makes bad documentation
+;; strings. Furthermore, to a `defun'ned function, a nil argument is
+;; the same as no argument, so it will use its (manual) default, but
+;; to a `cl-defun'ned function, a nil argument *is* the argument, so
+;; any default specified in the arglist will be ignored. Confusion of
+;; this type should be avoided when at all possible.
 
 ;;}}}
 
 ;;; Code:
 
-(require 'cl)
+(require 'cl-lib)
 ;; If we don't load font-lock now, but it is loaded later, the
 ;; necessary mmm-font-lock-* properties may not be there.
 (require 'font-lock)
@@ -123,6 +123,16 @@ submode region, the following changes \(are supposed to) occur:
 4. Some local variables of the submode shadow the default mode's.
 5. Font-lock fontifies correctly for the submode.
 6. Indentation function dispatches to the appropriate submode.
+7. User-specified hooks run when the point enters or exits a submode.
+
+The user may specify a number of hooks which may run when the point
+transitions between submodes, or between the primary mode and a
+submode. When a major mode is entered, the hook mmm-x-enter-hook is
+run, where x is the name of the major mode. When a major mode is
+left, the hook mmm-y-exit-hook is run, where y is the name of the
+major mode. Users may also specify hooks with names of the form
+mmm-x-y-hook which are run when the point leaves a mode named x,
+and enters a mode named y.
 
 For further information, including installation and configuration
 instructions, see the Info file mmm.info which is included with the
@@ -153,7 +163,7 @@ available through M-x customize-group RET mmm."
      (mmm-update-mode-info major-mode)
      (setq mmm-region-saved-locals-for-dominant
            ;; FIXME: Neither is defined in recent Emacs.
-           (list* (list 'font-lock-cache-state nil)
+           (cl-list* (list 'font-lock-cache-state nil)
                   (list 'font-lock-cache-position (make-marker))
                   (copy-tree (cdr (assq major-mode mmm-region-saved-locals-defaults)))))
      ;; Without the next line, the (make-marker) above gets replaced
@@ -161,11 +171,12 @@ available through M-x customize-group RET mmm."
      (mmm-set-local-variables major-mode nil)
      (mmm-add-hooks)
      (mmm-fixup-skeleton)
-     (make-local-variable 'font-lock-fontify-region-function)
-     (setq font-lock-fontify-region-function 'mmm-fontify-region)
-     (set (make-local-variable 'syntax-begin-function) nil)
+     (set (make-local-variable 'font-lock-fontify-region-function)
+          #'mmm-fontify-region)
+     (when (boundp 'syntax-begin-function)
+       (set (make-local-variable 'syntax-begin-function) nil))
      (set (make-local-variable 'syntax-propertize-function)
-          'mmm-syntax-propertize-function)
+          #'mmm-syntax-propertize-function)
      (set (make-local-variable 'indent-line-function) mmm-indent-line-function)
      (setq mmm-mode t)
      (condition-case err
@@ -194,8 +205,6 @@ available through M-x customize-group RET mmm."
     (mmm-update-submode-region)
     (setq font-lock-fontify-region-function
           (get mmm-primary-mode 'mmm-fontify-region-function))
-    (set 'syntax-begin-function
-         (get mmm-primary-mode 'mmm-beginning-of-syntax-function))
     (mmm-update-font-lock-buffer)
     (mmm-refontify-maybe)
     (setq mmm-mode nil)
