@@ -1,9 +1,8 @@
 ;;; mmm-region.el --- Manipulating and behavior of MMM submode regions
 
-;; Copyright (C) 2000 by Michael Abraham Shulman
-;; Copyright (C) 2012, 2013 by Dmitry Gutov
+;; Copyright (C) 2000-2003, 2010-2015, 2018  Free Software Foundation, Inc.
 
-;; Author: Michael Abraham Shulman <viritrilbia@users.sourceforge.net>
+;; Author: Michael Abraham Shulman <viritrilbia@gmail.com>
 
 ;;{{{ GPL
 
@@ -36,7 +35,7 @@
 
 ;;; Code:
 
-(require 'cl)
+(require 'cl-lib)
 (require 'font-lock)
 (require 'mmm-compat)
 (require 'mmm-utils)
@@ -61,8 +60,8 @@ See `mmm-included-p' for the values of TYPE."
 See `mmm-included-p' for the values of TYPE."
   (or pos (setq pos (point)))
   (mmm-sort-overlays
-   (remove-if-not
-    #'(lambda (ovl)
+   (cl-remove-if-not
+    (lambda (ovl)
 	(and (overlay-get ovl 'mmm)
 	     (mmm-included-p ovl pos type)))
     ;; XEmacs complains about positions outside the buffer
@@ -82,40 +81,40 @@ should be one of nil, `beg', `end', `none', or `all'.
 * If TYPE is `end', return true for any overlay ending at POS but
   false for any starting at POS.
 * If TYPE is `all', return true for any overlay starting or ending at POS.
-* If TYPE is `none' \(or any other value), return false for any
+* If TYPE is `none' (or any other value), return false for any
   overlay starting or ending at POS."
   (let ((beg (overlay-start ovl))
 	(end (overlay-end ovl)))
     (cond ((and (= beg pos) (= end pos))
 	   ;; Do the Right Thing for zero-width overlays
-	   (case type
+	   (cl-case type
 	     ((nil) (and (overlay-get ovl 'beg-sticky)
 			 (overlay-get ovl 'end-sticky)))
 	     ((none) nil)
 	     (t t)))
 	  ((= beg pos)
-	   (case type
+	   (cl-case type
 	     ((nil) (overlay-get ovl 'beg-sticky))
 	     ((beg all) t)
 	     (t nil)))
 	  ((= end pos)
-	   (case type
+	   (cl-case type
 	     ((nil) (overlay-get ovl 'end-sticky))
 	     ((end all) t)
 	     (t nil)))
 	  ((and (> end pos) (< beg pos))
 	   t))))
 
-;;; `mmm-overlays-in' has been retired as altogether too confusing a
-;;; name, when what is really meant is one of the following three:
+;; `mmm-overlays-in' has been retired as altogether too confusing a
+;; name, when what is really meant is one of the following three:
 
 (defun mmm-overlays-containing (start stop)
   "Return all MMM overlays containing the region START to STOP.
 The overlays are returned in order of decreasing priority.  No
 attention is paid to stickiness."
   (mmm-sort-overlays
-   (remove-if-not
-    #'(lambda (ovl)
+   (cl-remove-if-not
+    (lambda (ovl)
 	(and (overlay-get ovl 'mmm)
 	     (<= (overlay-start ovl) start)
 	     (>= (overlay-end ovl) stop)))
@@ -127,8 +126,8 @@ attention is paid to stickiness."
 The overlays are returned in order of decreasing priority.  No
 attention is paid to stickiness."
   (mmm-sort-overlays
-   (remove-if-not
-    #'(lambda (ovl)
+   (cl-remove-if-not
+    (lambda (ovl)
 	(and (overlay-get ovl 'mmm)
 	     (>= (overlay-start ovl) start)
 	     (<= (overlay-end ovl) stop)))
@@ -140,16 +139,16 @@ attention is paid to stickiness."
 The overlays are returned in order of decreasing priority.  No
 attention is paid to stickiness."
   (mmm-sort-overlays
-   (remove-if-not
-    #'(lambda (ovl)
+   (cl-remove-if-not
+    (lambda (ovl)
 	(overlay-get ovl 'mmm))
     (overlays-in (max start (point-min))
 		 (min stop (point-max))))))
 
 (defun mmm-sort-overlays (overlays)
   "Sort OVERLAYS in order of decreasing priority."
-  (sort (copy-list overlays)
-        #'(lambda (x y) (> (or (overlay-get x 'priority) 0)
+  (sort (cl-copy-list overlays)
+        (lambda (x y) (> (or (overlay-get x 'priority) 0)
                            (or (overlay-get y 'priority) 0)))))
 
 ;;}}}
@@ -182,7 +181,7 @@ Return non-nil if the current region changed.
 Also deletes overlays that ought to evaporate because their delimiters
 have disappeared."
   (mapc #'delete-overlay
-	(remove-if #'(lambda (ovl)
+	(cl-remove-if (lambda (ovl)
 		       (or (not (eq (overlay-get ovl 'mmm-evap) 'front))
 			   (overlay-buffer (overlay-get ovl 'front))))
 		   (mmm-overlays-at pos)))
@@ -276,7 +275,7 @@ a valid child of the highest-priority of those regions, if any.
 Signals errors, returns `t' if no error."
   ;; First check if the placement is valid.  Every existing region
   ;; that overlaps this one must contain it in its entirety.
-  (let ((violators (set-difference
+  (let ((violators (cl-set-difference
 		    (mmm-overlays-overlapping beg end)
 		    (mmm-overlays-containing beg end))))
     (if violators
@@ -295,7 +294,7 @@ Signals errors, returns `t' if no error."
 		 (list parent-mode))))
   t)
 
-(defun* mmm-make-region
+(cl-defun mmm-make-region
     (submode beg end &key face
 	     front back (evaporation 'front)
 	     delimiter-mode front-face back-face
@@ -428,13 +427,13 @@ with point at the start of the new region."
 Does not handle delimiters.  Use `mmm-make-region'."
   (let ((ovl (make-overlay beg end nil (not beg-sticky) end-sticky)))
     (mapc
-     #'(lambda (pair) (overlay-put ovl (car pair) (cadr pair)))
+     (lambda (pair) (overlay-put ovl (car pair) (cadr pair)))
      `((mmm t)				; Mark all submode overlays
        (mmm-mode ,submode)
        ,@(if delim '((delim t)) nil)
        (mmm-local-variables
 	;; Have to be careful to make new list structure here
-	,(list* (list 'font-lock-cache-state nil)
+	,(cl-list* (list 'font-lock-cache-state nil)
 		(list 'font-lock-cache-position (make-marker))
 		(copy-tree
 		 (cdr (assq submode mmm-region-saved-locals-defaults)))))
@@ -514,8 +513,6 @@ is non-nil, don't quit if the info is already there."
                 ;; original values elsewhere.
                 (put mode 'mmm-fontify-region-function
                      font-lock-fontify-region-function)
-                (put mode 'mmm-beginning-of-syntax-function
-                     syntax-begin-function)
                 (put mode 'mmm-syntax-propertize-function
                      (and (boundp 'syntax-propertize-function)
                           syntax-propertize-function))
@@ -545,10 +542,14 @@ different keymaps, syntax tables, local variables, etc. for submodes."
   (when (mmm-update-current-submode)
     (mmm-save-changed-local-variables mmm-previous-submode
                                       mmm-previous-overlay)
-    (let ((mode (or mmm-current-submode mmm-primary-mode)))
-      (mmm-update-mode-info mode)
-      (mmm-set-local-variables mode mmm-current-overlay)
-      (mmm-enable-font-lock mode))
+    (let ((new-mode (or mmm-current-submode mmm-primary-mode))
+          (old-mode (or mmm-previous-submode mmm-primary-mode)))
+      (mmm-run-constructed-hook old-mode new-mode)
+      (mmm-run-constructed-hook old-mode "exit")
+      (mmm-run-constructed-hook new-mode "enter")
+      (mmm-update-mode-info new-mode)
+      (mmm-set-local-variables new-mode mmm-current-overlay)
+      (mmm-enable-font-lock new-mode))
     (mmm-set-mode-line)
     (dolist (func (if mmm-current-overlay
 		      (overlay-get mmm-current-overlay 'entry-hook)
@@ -558,15 +559,15 @@ different keymaps, syntax tables, local variables, etc. for submodes."
 (defun mmm-add-hooks ()
   (if (featurep 'xemacs)
       (make-local-hook 'post-command-hook))
-  (add-hook 'post-command-hook 'mmm-update-submode-region nil t)
+  (add-hook 'post-command-hook #'mmm-update-submode-region nil t)
   (when mmm-parse-when-idle
-    (add-hook 'pre-command-hook 'mmm-mode-reset-timer nil t)
-    (add-hook 'after-change-functions 'mmm-mode-edit nil t)))
+    (add-hook 'pre-command-hook #'mmm-mode-reset-timer nil t)
+    (add-hook 'after-change-functions #'mmm-mode-edit nil t)))
 
 (defun mmm-remove-hooks ()
-  (remove-hook 'post-command-hook 'mmm-update-submode-region t)
-  (remove-hook 'pre-command-hook 'mmm-mode-reset-timer t)
-  (remove-hook 'after-change-functions 'mmm-mode-edit t))
+  (remove-hook 'post-command-hook #'mmm-update-submode-region t)
+  (remove-hook 'pre-command-hook #'mmm-mode-reset-timer t)
+  (remove-hook 'after-change-functions #'mmm-mode-edit t))
 
 ;;}}}
 ;;{{{ Local Variables
@@ -575,7 +576,7 @@ different keymaps, syntax tables, local variables, etc. for submodes."
   "Filter `mmm-save-local-variables' to match TYPE and MODE.
 Return a list \(VAR ...).  In some cases, VAR will be a cons cell
 \(GETTER . SETTER) -- see `mmm-save-local-variables'."
-  (mapcan #'(lambda (element)
+  (mmm-mapcan (lambda (element)
               (and (if (and (consp element)
                             (cdr element)
                             (cadr element))
@@ -583,10 +584,10 @@ Return a list \(VAR ...).  In some cases, VAR will be a cons cell
                      (eq type 'global))
                    (if (and (consp element)
                             (cddr element)
-                            (not (eq (caddr element) t)))
-                       (if (functionp (caddr element))
-                           (funcall (caddr element))
-                         (member mode (caddr element)))
+                            (not (eq (nth 2 element) t)))
+                       (if (functionp (nth 2 element))
+                           (funcall (nth 2 element))
+                         (member mode (nth 2 element)))
                      t)
                    (list (if (consp element) (car element) element))))
           mmm-save-local-variables))
@@ -595,7 +596,7 @@ Return a list \(VAR ...).  In some cases, VAR will be a cons cell
   "Get the local variables and values for TYPE from this buffer.
 Return \((VAR VALUE) ...).  In some cases, VAR will be of the form
 \(GETTER . SETTER) -- see `mmm-save-local-variables'."
-  (mapcan #'(lambda (var)
+  (mmm-mapcan (lambda (var)
               (if (consp var)
                   `((,var ,(funcall (car var))))
                 (and (boundp var)
@@ -617,7 +618,7 @@ Return \((VAR VALUE) ...).  In some cases, VAR will be of the form
   "Set all the local variables saved for MODE and OVL.
 Looks up global, buffer and region saves.  When MODE is nil, just
 the region ones."
-  (mapcar #'(lambda (var)
+  (mapcar (lambda (var)
               ;; (car VAR) may be (GETTER . SETTER)
               (if (consp (car var))
                   (funcall (cdar var) (cadr var))
@@ -657,7 +658,7 @@ region and mode for the previous position."
                          (overlay-get ovl 'mmm-local-variables)
                        mmm-region-saved-locals-for-dominant))
         (set-local-value
-         #'(lambda (var)
+         (lambda (var)
              (setcar (cdr var)
                      ;; (car VAR) may be (GETTER . SETTER)
                      (if (consp (car var))
@@ -686,36 +687,44 @@ region and mode for the previous position."
 
 (defun mmm-update-font-lock-buffer ()
   "Turn on font lock if any mode in the buffer enables it."
-  (if (some #'(lambda (mode)
+  (if (cl-some (lambda (mode)
                 (get mode 'mmm-font-lock-mode))
             (cons mmm-primary-mode
-                  (mapcar #'(lambda (ovl)
+                  (mapcar (lambda (ovl)
                               (overlay-get ovl 'mmm-mode))
                           (mmm-overlays-overlapping
                            (point-min) (point-max)))))
       (font-lock-mode 1)
     (font-lock-mode 0)))
 
+;; FIXME: Most uses of this function happen when the buffer is parsed
+;; into regions manually. That should go away after
+;; syntax-propertize-function does this.
 (defun mmm-refontify-maybe (&optional start stop)
   "Re-fontify from START to STOP, or entire buffer, if enabled."
-  (and font-lock-mode
-       (if (or start stop)
-           (font-lock-fontify-region (or start (point-min))
-                                     (or stop (point-max)))
-         (font-lock-fontify-buffer))))
+  (when font-lock-mode
+    (if (fboundp 'font-lock-flush)
+        (progn
+          (font-lock-flush start stop)
+          ;; FIXME: Do we really need to do this eagerly here?
+          (font-lock-ensure start stop))
+      (if (or start stop)
+          (font-lock-fontify-region (or start (point-min))
+                                    (or stop (point-max)))
+        (with-no-warnings (font-lock-fontify-buffer))))))
 
 ;;}}}
 ;;{{{ Get Submode Regions
 
-;;; In theory, these are general functions that have nothing to do
-;;; with font-lock, but they aren't used anywhere else, so we might as
-;;; well have them close.
+;; In theory, these are general functions that have nothing to do
+;; with font-lock, but they aren't used anywhere else, so we might as
+;; well have them close.
 
 (defun mmm-submode-changes-in (start stop)
   "Return a list of all submode-change positions from START to STOP.
 The list is sorted in order of increasing buffer position."
-  (let ((changes (sort (remove-duplicates
-                        (mapcan #'(lambda (ovl)
+  (let ((changes (sort (cl-remove-duplicates
+                        (mmm-mapcan (lambda (ovl)
                                     `(,(overlay-start ovl)
                                       ,(overlay-end ovl)))
                                 (mmm-overlays-overlapping start stop)))
@@ -732,7 +741,7 @@ The list is sorted in order of increasing buffer position."
 union covers the region from START to STOP, including delimiters."
   (when (> stop start)
     (let ((regions
-           (maplist #'(lambda (pos-list)
+           (cl-maplist (lambda (pos-list)
                         (when (cdr pos-list)
                           (let ((ovl (mmm-overlay-at (car pos-list) 'beg)))
                             (list (if ovl
@@ -776,7 +785,7 @@ of the REGIONS covers START to STOP."
           ;; Necessary to catch changes in font-lock cache state and position.
           (mmm-save-changed-local-variables
            mmm-current-submode mmm-current-overlay)
-          (mapc #'(lambda (elt)
+          (mapc (lambda (elt)
                     (when (get (car elt) 'mmm-font-lock-mode)
                       (mmm-fontify-region-list (car elt) (cdr elt))))
                 (mmm-regions-alist start stop)))
@@ -786,13 +795,16 @@ of the REGIONS covers START to STOP."
       (mmm-set-local-variables (or saved-mode mmm-primary-mode) saved-ovl)))
   (when loudly (message nil)))
 
+(defvar syntax-ppss-cache)
+(defvar syntax-ppss-last)
+
 (defun mmm-fontify-region-list (mode regions)
-  "Fontify REGIONS, each like \(BEG END), in mode MODE."
+  "Fontify REGIONS, each like (BEG END), in mode MODE."
   (save-excursion
     (let ((func (get mode 'mmm-fontify-region-function))
           font-lock-extend-region-functions)
-      (mapc #'(lambda (reg)
-                (destructuring-bind (beg end ovl) reg
+      (mapc (lambda (reg)
+                (cl-destructuring-bind (beg end ovl) reg
                   (goto-char beg)
                   ;; Here we do the same sort of thing that
                   ;; `mmm-update-submode-region' does, but we force it
@@ -804,8 +816,13 @@ of the REGIONS covers START to STOP."
                                            mmm-current-overlay)
                   (save-restriction
                     (let ((font-lock-dont-widen t)
+                          ;; FIXME: Messing with syntax-ppss-* vars should not
+                          ;; be needed any more in Emacs≥26.
                           syntax-ppss-last syntax-ppss-cache)
-                      (when ovl (narrow-to-region beg end))
+                      ;; TODO: Remove this conditional when cc-mode
+                      ;; respects submode boundaries.
+                      (when (and ovl (not (memq mode mmm-c-derived-modes)))
+                        (narrow-to-region beg end))
                       (funcall func beg end nil)))
                   ;; Catch changes in font-lock cache.
                   (mmm-save-changed-local-variables
@@ -829,11 +846,13 @@ calls each respective submode's `syntax-propertize-function'."
     (mmm-save-changed-local-variables
      mmm-current-submode mmm-current-overlay)
     (unwind-protect
-        (mapc #'(lambda (elt)
+        (mapc (lambda (elt)
                   (let* ((mode (car elt))
                          (func (get mode 'mmm-syntax-propertize-function))
-                         (beg (cadr elt)) (end (caddr elt))
-                         (ovl (cadddr elt))
+                         (beg (cadr elt)) (end (nth 2 elt))
+                         (ovl (nth 3 elt))
+                         ;; FIXME: Messing with syntax-ppss-* vars should not
+                         ;; be needed any more in Emacs≥26.
                          syntax-ppss-cache
                          syntax-ppss-last)
                     (goto-char beg)
@@ -858,16 +877,38 @@ calls each respective submode's `syntax-propertize-function'."
 ;;}}}
 ;;{{{ Indentation
 
-(defvar mmm-indent-line-function 'mmm-indent-line
+(defvar mmm-indent-line-function #'mmm-indent-line
   "The function to call to indent a line.
 This will be the value of `indent-line-function' for the whole
 buffer. It's supposed to delegate to the appropriate submode's
 indentation function. See `mmm-indent-line' as the starting point.")
 
+(defun mmm-indent-line-narrowed ()
+  "An indent function which works on some modes where `mmm-indent-line' doesn't.
+Works like `mmm-indent-line', but narrows the buffer before indenting to
+appease modes which rely on constructs like (point-min) to indent."
+  (interactive)
+  (funcall
+   (save-excursion
+     (back-to-indentation)
+     (mmm-update-submode-region)
+     (let ((indent-function (get
+                             (if (and mmm-current-overlay
+                                      (> (overlay-end mmm-current-overlay) (point)))
+                                 mmm-current-submode
+                               mmm-primary-mode)
+                             'mmm-indent-line-function)))
+       (if mmm-current-overlay
+           (save-restriction
+             (narrow-to-region (overlay-start mmm-current-overlay)
+                               (overlay-end mmm-current-overlay))
+             indent-function)
+         indent-function)))))
+
 (defun mmm-indent-line ()
   (interactive)
   (funcall
-    (save-excursion
+   (save-excursion
       (back-to-indentation)
       (mmm-update-submode-region)
       (get
