@@ -1,9 +1,8 @@
 ;;; mmm-vars.el --- Variables for MMM Mode
 
-;; Copyright (C) 2000, 2004 by Michael Abraham Shulman
-;; Copyright (C) 2012, 2013 by Dmitry Gutov
+;; Copyright (C) 2000-2004, 2011-2015, 2018  Free Software Foundation, Inc.
 
-;; Author: Michael Abraham Shulman <viritrilbia@users.sourceforge.net>
+;; Author: Michael Abraham Shulman <viritrilbia@gmail.com>
 
 ;;{{{ GPL
 
@@ -33,22 +32,26 @@
 ;;; Code:
 
 (require 'mmm-compat)
-(require 'cl)
+(require 'mmm-utils)
+(require 'cl-lib)
 
 ;; MISCELLANEOUS
 ;;{{{ Shut up the Byte Compiler
 
 ;; Otherwise it complains about undefined variables.
-(eval-when-compile
-  (defvar mmm-current-submode)
-  (defvar mmm-save-local-variables)
-  (defvar mmm-mode-string)
-  (defvar mmm-submode-mode-line-format)
-  (defvar mmm-mode-ext-classes-alist)
-  (defvar mmm-mode-prefix-key)
-  (defvar mmm-global-mode)
-  (defvar mmm-primary-mode)
-  (defvar mmm-classes-alist))
+(defvar mmm-current-submode)
+(defvar mmm-save-local-variables)
+(defvar mmm-mode-string)
+(defvar mmm-submode-mode-line-format)
+(defvar mmm-mode-ext-classes-alist)
+(defvar mmm-mode-prefix-key)
+(defvar mmm-global-mode)
+(defvar mmm-primary-mode)
+(defvar mmm-classes-alist)
+(defvar mmm-current-overlay)
+(declare-function mmm-apply-all "mmm-class")
+(declare-function mmm-set-class-parameter "mmm-class" (class param value))
+(declare-function mmm-get-class-parameter "mmm-class" (class param))
 
 ;;}}}
 ;;{{{ Error Conditions
@@ -108,7 +111,7 @@
   '(c-mode c++-mode objc-mode pike-mode java-mode jde-mode javascript-mode
     php-mode))
 
-(defvar mmm-save-local-variables 
+(defvar mmm-save-local-variables
   `(;; Don't use `function' (#') here!!  We're already inside `quote'!
     major-mode
     comment-start
@@ -131,6 +134,7 @@
     font-lock-syntax-table
     font-lock-mark-block-function       ; Override this?
     font-lock-syntactic-keywords
+    font-lock-syntactic-face-function
     parse-sexp-ignore-comments  ; Fixes indentation in PHP-mode?
     ;; Can be different in different buffers
     (c-basic-offset
@@ -145,10 +149,11 @@
     ;; Indentation style control variables.
     ;; These have to be localized in Emacs: see `mmm-mode-on'.
     ,@(mapcar
-       #'(lambda (var) (list var nil mmm-c-derived-modes))
+       (lambda (var) (list var nil mmm-c-derived-modes))
        '(c++-template-syntax-table
 	 c-<-op-cont-regexp 
-	 c->-op-cont-regexp 
+	 c->-op-cont-regexp
+         c-after-brace-list-key
 	 c-after-suffixed-type-decl-key
 	 c-after-suffixed-type-maybe-decl-key
 	 c-any-class-key
@@ -156,15 +161,18 @@
 	 c-assignment-op-regexp
 	 c-backslash-column
 	 c-basic-offset
+         c-before-context-fontification-functions
 	 c-bitfield-kwds
 	 c-block-comment-prefix
 	 c-block-decls-with-vars
          c-block-prefix-charset
+         c-block-stmt-1-2-key
 	 c-block-stmt-1-key
 	 c-block-stmt-1-kwds
 	 c-block-stmt-2-key
 	 c-block-stmt-2-kwds
-	 c-brace-list-key 
+	 c-brace-list-key
+         c-case-kwds-regexp
 	 c-cast-parens 
 	 c-class-key
 	 c-class-kwds
@@ -186,6 +194,7 @@
 	 c-expr-kwds
 	 c-file-offsets
 	 c-file-style
+         c-not-primitive-type-keywords-regexp
 	 c-hanging-braces-alist
 	 c-hanging-colons-alist
 	 c-hanging-comment-ender-p
@@ -206,14 +215,14 @@
 	 c-keywords-obarray
 	 c-keywords-regexp
 	 c-known-type-key
-	 c-label-key
-	 c-label-kwds
+         c-label-kwds
 	 c-label-kwds-regexp
 	 c-label-minimum-indentation
 	 c-lambda-kwds
 	 c-literal-start-regexp 
 	 c-macro-with-semi-re
          c-nonlabel-token-key
+         c-nonlabel-token-2-key
 	 c-nonsymbol-chars 
 	 c-nonsymbol-token-regexp
 	 c-not-decl-init-keywords
@@ -252,6 +261,7 @@
 	 c-primitive-type-key 
 	 c-primitive-type-kwds
 	 c-protection-kwds
+         c-postfix-decl-spec-key
 	 c-recognize-<>-arglists 
 	 c-recognize-knr-p
 	 c-recognize-paren-inits 
@@ -264,6 +274,7 @@
 	 c-specifier-kwds
 	 c-stmt-delim-chars 
 	 c-stmt-delim-chars-with-comma
+         c-symbol-char-key
 	 c-symbol-key
 	 c-symbol-start 
 	 c-syntactic-eol
@@ -271,7 +282,8 @@
 	 c-syntactic-ws-start 
 	 c-type-decl-prefix-key 
 	 c-type-decl-suffix-key 
-	 c-type-prefix-key 
+	 c-type-prefix-key
+         c-typeof-key
          c-prefix-spec-kwds-re
          c-typedef-key
 	 c-typedef-decl-key
@@ -589,7 +601,7 @@ element in `mmm-major-mode-preferences'.  In the latter case, the
 first `fboundp' element of the `cdr' is returned, or nil if none."
   (if (fboundp mode)
       mode
-    (car (remove-if-not
+    (car (cl-remove-if-not
           #'fboundp
           (cdr (assq mode mmm-major-mode-preferences))))))
 
@@ -725,7 +737,7 @@ available so that others can take advantage of the hack as well.
 
 Note that file local variables have *not* been processed by the time
 this hook is run. If a function needs to inspect them, it should also
-be added to `find-file-hooks'. However, `find-file-hooks' is not run
+be added to `find-file-hook'. However, `find-file-hook' is not run
 when creating a non-file-based buffer, or when changing major modes in
 an existing buffer."
   :group 'mmm
@@ -738,14 +750,14 @@ an existing buffer."
 ;;}}}
 ;;{{{ MMM Global Mode
 
-;;; There's a point to be made that this variable should default to
-;;; `maybe' (i.e. not nil and not t), because that's what practically
-;;; everyone wants.  I subscribe, however, to the view that simply
-;;; *loading* a lisp extension should not change the (user-visible)
-;;; behavior of Emacs, until it is configured or turned on in some
-;;; way, which dictates that the default for this must be nil.
+;; There's a point to be made that this variable should default to
+;; `maybe' (i.e. not nil and not t), because that's what practically
+;; everyone wants.  I subscribe, however, to the view that simply
+;; *loading* a lisp extension should not change the (user-visible)
+;; behavior of Emacs, until it is configured or turned on in some
+;; way, which dictates that the default for this must be nil.
 (defcustom mmm-global-mode nil
-  "*Specify in which buffers to turn on MMM Mode automatically.
+  "Specify in which buffers to turn on MMM Mode automatically.
 
 - If nil, MMM Mode is never enabled automatically.
 - If t, MMM Mode is enabled automatically in all buffers.
@@ -804,7 +816,7 @@ than it solves, but some modes require it.")
 (defvar mmm-mode-buffer-dirty nil "Private variable.")
 (make-variable-buffer-local 'mmm-mode-buffer-dirty)
 
-(defun mmm-mode-edit (beg end len)
+(defun mmm-mode-edit (_beg _end _len)
   (setq mmm-mode-buffer-dirty t)
   (mmm-mode-reset-timer))
 
@@ -1011,28 +1023,28 @@ and is not for the user to see.")
 (defun mmm-add-group (group classes)
   "Add CLASSES and a \"grouping class\" named GROUP which calls them all.
 The CLASSES are all made private, i.e. non-user-visible."
-  (mmm-add-classes (mapcar #'(lambda (class)
+  (mmm-add-classes (mapcar (lambda (class)
                                (append class
                                        '(:private t)))
                            classes))
   (add-to-list 'mmm-classes-alist
-               (list group :classes (mapcar #'first classes))))
+               (list group :classes (mapcar #'cl-first classes))))
 
 (defun mmm-add-to-group (group classes)
   "Add CLASSES to the \"grouping class\" named GROUP.
 The CLASSES are all made private, i.e. non-user-visible."
-  (mmm-add-classes (mapcar #'(lambda (class)
+  (mmm-add-classes (mapcar (lambda (class)
                                (append class
                                        '(:private t)))
                            classes))
   (mmm-set-class-parameter group :classes
 			   (append  (mmm-get-class-parameter group :classes)
-				    (mapcar #'first classes))))
+				    (mapcar #'cl-first classes))))
 
 ;;}}}
 ;;{{{ Version Number
 
-(defconst mmm-version "0.5.4"
+(defconst mmm-version "0.5.6"
   "Current version of MMM Mode.")
 
 (defun mmm-version ()
@@ -1080,8 +1092,8 @@ Set automatically from `mmm-mode-ext-classes-alist'.")
 Uses `mmm-mode-ext-classes-alist' to find submode classes."
   (or mmm-mode-ext-classes
       (setq mmm-mode-ext-classes
-            (mapcar #'third
-                    (remove-if-not #'mmm-mode-ext-applies
+            (mapcar #'cl-third
+                    (cl-remove-if-not #'mmm-mode-ext-applies
                                    mmm-mode-ext-classes-alist)))))
 
 (defun mmm-clear-mode-ext-classes ()
@@ -1089,7 +1101,7 @@ Uses `mmm-mode-ext-classes-alist' to find submode classes."
   (setq mmm-mode-ext-classes nil))
 
 (defun mmm-mode-ext-applies (element)
-  (destructuring-bind (mode ext class) element
+  (cl-destructuring-bind (mode ext _class) element
     (and (if mode
              (eq mode
                  ;; If MMM is on in this buffer, use the primary mode,
